Attribute VB_Name = "modFileData"
'RUMINT, A PVR for network traffic
'Copyright (C) 2005 Greg Conti

'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; either version 2
'of the License, or (at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.



'modFileData
'
'The modFileData module contains subroutines for handling all
'of rumint's file handling operations.

Option Explicit


'This subroutine allows the user to select the file they
'wish to open.  It then calls the subroutine that will load
'and parse the contents
Public Sub LoadData()
  frmLoadDataset.cdlFileOpen.DialogTitle = "Open rumint dataset"
  frmLoadDataset.cdlFileOpen.InitDir = App.Path 'for future this should point to user preferred data directory
  frmLoadDataset.cdlFileOpen.ShowOpen 'code stops here and waits...
  If (frmLoadDataset.cdlFileOpen.FileName <> "") Then 'make sure a file was selected
    Call OpenFile
  End If
End Sub

'SaveData is a stub subroutine that will eventually handle the saving of datasets
Public Sub SaveData()
  'Debug.Print "saving data..."
  frmSaveDataset.Show
End Sub

'This subroutine opens the rumint formatted file and parses it into individual packets
Private Sub OpenFile()

  'Open frmLoadDataset.cdlFileOpen.FileName For Binary As #1 'open the selected file
  
  Dim temploopcounter As Long 'a generic loop counter
  Dim tempbyte As Byte 'holds current byte
  Dim templong As Long 'used for multiplication
  Dim capLen As Long 'packet capture length
  Dim fileindex As Long 'current byteposition in file
  Dim fileLen As Long 'length of file
  Dim numPackets As Long 'number of packets in file
  Dim packedBytePacket(0 To 1600) As Long 'holds packed byte data, needed for packetx compatibility, assumes ether RFC's 1518 byte frame limit
  Dim firstByte As Byte 'D4 for little endian and A1 for big endian
  
  Call modLibrary.Timer("start", -1) 'start the optimization timer
  Open frmLoadDataset.cdlFileOpen.FileName For Binary As #1 'open the selected file
  
  fileLen = LOF(1)
  
  frmLoadDataset.lblFileLoadStatusVariable = "In Progress"
  frmLoadDataset.lblFileSizeVariable = fileLen
  frmLoadDataset.pbProcessing.Min = 0
  frmLoadDataset.pbProcessing.Max = fileLen
  frmLoadDataset.lblFileVariable = frmLoadDataset.cdlFileOpen.FileTitle
  
  frmLoadDataset.Show
    
  'parse pcap file header
  'magic=4B versionMajor=2B versionMinor=2B, zone=4B, sigfigs=4B, snaplen=4B, linktype=4B
  'Debug.Print "pcap file header" 'format can be found in pcap.h
  Get #1, 1, firstByte 'used to store first byte of magic number to determine endianess
  For temploopcounter = 1 To 24
    Get #1, temploopcounter, tempbyte
    'Debug.Print " " & Hex(Str(tempbyte));
  Next
    
  fileindex = temploopcounter
  
  'todo:  perform safety check using magic number?
  modVisualize.clearAllVisualizationWindows
  Call modLibrary.initBuffer("file")  'initialize buffer
  
  Do While (fileindex < fileLen)
    
  numPackets = numPackets + 1
  If (numPackets >= modLibrary.MAXPACKETBUFFER) Then
    MsgBox ("Packets in file exceeded buffer.  Only the first " & modLibrary.MAXPACKETBUFFER & " packets will be used.")
    Exit Do 'jump out of the loop
  End If
                                        
  
  frmLoadDataset.pbProcessing.value = fileindex
  
  
  'parse pcap packet header
  'ts=8B (struct with seconds and useconds see time.h) caplen=4B len=4Bts
  'Debug.Print vbCrLf & vbCrLf & "pcap packet header" 'format can be found in pcap.h
  For temploopcounter = fileindex To fileindex + 15
    Dim temphigh, templow As Long
    
    Get #1, temploopcounter, tempbyte
    
    'little endian based on first byte
    If firstByte = 212 Then 'D4 e.g. little endian
      'the offset of the least significant byte of snaplen
      If temploopcounter = fileindex + 8 Then
        capLen = tempbyte
        'Debug.Print "first byte = " & tempbyte & " " & Hex(tempbyte)
      End If
      'the offset of the second least significant byte of snaplen, ignore rest
      If temploopcounter = fileindex + 9 Then
        templong = tempbyte
        'Debug.Print "second byte = " & tempbyte & " " & Hex(tempbyte)
        capLen = (templong * 256) + capLen
        'Debug.Print "caplen = " & capLen
        'Debug.Print "(" & numPackets & ")=" & capLen & " "
        'Debug.Print " " & Hex(Str(tempbyte));
      End If
    Else
      If firstByte = 161 Then 'A1 e.g. big endian
        'the offset of the least significant byte of snaplen
        If temploopcounter = fileindex + 11 Then
          capLen = tempbyte
          'Debug.Print "first byte = " & tempbyte & " " & Hex(tempbyte)
        End If
        'the offset of the second least significant byte of snaplen, ignore rest
        If temploopcounter = fileindex + 10 Then
          templong = tempbyte
          'Debug.Print "second byte = " & tempbyte & " " & Hex(tempbyte)
          capLen = (templong * 256) + capLen
          'Debug.Print "caplen = " & capLen
          'Debug.Print "(" & numPackets & ")=" & capLen & " "
          'Debug.Print " " & Hex(Str(tempbyte));
        End If
      Else 'unknown endianess
        MsgBox ("Unable to parse file format.  ErrorCode=" & Hex(firstByte))
        Close #1
        End 'end program?
      End If
    End If
  Next
  
  DoEvents 'allow Windows to do housekeeping events (or interface may not refresh)
  fileindex = temploopcounter
  
  'parse pcap packet
  'variable length
  'Debug.Print vbCrLf & vbCrLf & "pcap packet"
  For temploopcounter = fileindex To fileindex + (capLen - 1)
    Get #1, temploopcounter, tempbyte
    packedBytePacket(temploopcounter - fileindex) = tempbyte 'fill array of current packet bytes
    'Debug.Print " " & Hex(Str(tempbyte));
  Next
  'Debug.Print vbCrLf & vbCrLf
  
      
  Call modLibrary.putPacket(packedBytePacket, capLen)
  fileindex = temploopcounter
    
  Loop 'while
      
  Close #1 'close pcap file

  frmFilters.initFilters 'set all filters based on this dataset
  frmLoadDataset.lblFileLoadStatusVariable = "Completed"
  frmLoadDataset.Hide 'file has been loaded and processed, hide progress window
  'Call modVisualize.selectedPlot(1, numPackets) 'this disables autoplay
  Call modLibrary.Timer("stop", numPackets) 'stop the optimization timer
  
End Sub
